/*
 * Copyright (c) Joe Maimon, jmaimon@jmaimon.com New York,NY 2005
 *
 * This milter is adapted from the sample milter included in the
 * open source sendmail distribution libmilter/README
 *
 * Portions may be copyright the Sendmail Consortium.
 *
 * No license was explicitly stated for that sample. In the event
 * that a License does apply, and this work is a significant deriviation
 * of the sample milter, the assumed license would be the license found
 * in the open source sendmail distribution LICENSE file.
 *
 * Additionaly, unless you received this file from all copyright holders under
 * any other arrangement, the GNU GPLv2 or later license may be assumed to apply, 
 * as specified by the Free Software Foundation.
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

#define _FFR_MILTER_REWRITE	1
#include "libmilter/mfapi.h"

#ifndef true
typedef int bool;
# define false	0
# define true	1
#endif /* ! true */

struct mlfiPriv
{
	char * rewrite_val; 
	int occurences;
};

#define MLFIPRIV	((struct mlfiPriv *) smfi_getpriv(ctx))

extern sfsistat	 mlfi_cleanup(SMFICTX *, bool);

sfsistat
mlfi_envfrom(ctx, envfrom)
	SMFICTX *ctx;
	char **envfrom;
{
	struct mlfiPriv *priv;

	/* allocate some private memory */
	priv = malloc(sizeof *priv);
	if (priv == NULL)
	{
		/* can't accept this message right now */
		return SMFIS_TEMPFAIL;
	}
	memset(priv, '\0', sizeof *priv);

	/* save the private data */
	smfi_setpriv(ctx, priv);

	/* continue processing */
	return SMFIS_CONTINUE;
}

sfsistat
mlfi_header(ctx, headerf, headerv)
	SMFICTX *ctx;
	char *headerf;
	char *headerv;
{

	if(!strcasecmp("From", headerf))
	{
		struct mlfiPriv *priv = MLFIPRIV;
		char *pbuf[2];
		char **pp = pbuf;
		char *tp = NULL; 
		int rwstat = 0;

		pbuf[0] = headerv;
		pbuf[1] = NULL;

		priv->occurences++;
			
		if((smfi_rewrite(ctx, "milter_rewrite_sender", &pp, &rwstat, 0)) == MI_SUCCESS)  
		{
			int i = 0;
			while(pp && pp[i])
			{
				if(!tp && rwstat < 0)
				{
					tp = pp[i++];
					continue;
				}
				free(pp[i++]);
			}
			if (pp)
				free(pp);
		}
		if(tp)
		{
			if(priv->rewrite_val)
			{
				free(priv->rewrite_val);
				priv->rewrite_val = NULL;
			}
			priv->rewrite_val = tp;
		}

	}
			
	/* continue processing */
	return SMFIS_CONTINUE;
}

sfsistat
mlfi_eoh(ctx)
	SMFICTX *ctx;
{

	struct mlfiPriv *priv = MLFIPRIV;
	if(priv->occurences == 0)
	{
		char *pbuf[2];
		char **pp = pbuf;
		char *tp = NULL;
		int rwstat = 0;

		pbuf[0] = "";
		pbuf[1] = NULL;

			
		if((smfi_rewrite(ctx, "milter_rewrite_sender", &pp, &rwstat, 0)) == MI_SUCCESS)  
		{
			int i = 0;
			while(pp && pp[i])
			{
				if(!tp && rwstat < 0)
				{
					tp = pp[i++];
					continue;
				}
				free(pp[i++]);
			}
			if (pp)
				free(pp);
		}
		if(tp)
		{
			if(priv->rewrite_val)
			{
				free(priv->rewrite_val);
				priv->rewrite_val = NULL;
			}
			priv->rewrite_val = tp;
		}
	}


	
	/* continue processing */
	return SMFIS_CONTINUE;
}

sfsistat
mlfi_body(ctx, bodyp, bodylen)
	SMFICTX *ctx;
	u_char *bodyp;
	size_t bodylen;
{

	/* continue processing */
	return SMFIS_CONTINUE;
}

sfsistat
mlfi_eom(ctx)
	SMFICTX *ctx;
{
	struct mlfiPriv *priv = MLFIPRIV;
	if (priv == NULL)
		return SMFIS_CONTINUE;
	if(priv->rewrite_val)
	{
		if(priv->occurences <= 0 )
			smfi_addheader(ctx, "From", priv->rewrite_val);
		else
			while(priv->occurences > 0)
				smfi_chgheader(ctx, "From", priv->occurences--, priv->rewrite_val);
	}
		

	return mlfi_cleanup(ctx, true);
}

sfsistat
mlfi_close(ctx)
	SMFICTX *ctx;
{
	return SMFIS_ACCEPT;
}

sfsistat
mlfi_abort(ctx)
	SMFICTX *ctx;
{
	return mlfi_cleanup(ctx, false);
}

sfsistat
mlfi_cleanup(ctx, ok)
	SMFICTX *ctx;
	bool ok;
{
	sfsistat rstat = SMFIS_CONTINUE;
	struct mlfiPriv *priv = MLFIPRIV;
	if (priv == NULL)
		return rstat;
	

	/* release private memory */
	if(priv->rewrite_val)
		free(priv->rewrite_val);
	free(priv);
	smfi_setpriv(ctx, NULL);

	/* return status */
	return rstat;
}

struct smfiDesc smfilter =
{
	"milter-rewrite-sender",	/* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	SMFIF_ADDHDRS,	/* flags */
	NULL,		/* connection info filter */
	NULL,		/* SMTP HELO command filter */
	mlfi_envfrom,	/* envelope sender filter */
	NULL,		/* envelope recipient filter */
	mlfi_header,	/* header filter */
	mlfi_eoh,	/* end of header */
	NULL,	/* body block filter */
	mlfi_eom,	/* end of message */
	mlfi_abort,	/* message aborted */
	mlfi_close	/* connection cleanup */
};


int
main(argc, argv)
	int argc;
	char *argv[];
{
	bool setconn = false;
	int c;
	const char *args = "d:p:";

	/* Process command line options */
	while ((c = getopt(argc, argv, args)) != -1)
	{
		switch (c)
		{
                  case 'd':
			smfi_setdbg(atoi(optarg));
			break;
		  case 'p':
			if (optarg == NULL || *optarg == '\0')
			{
				(void) fprintf(stderr, "Illegal conn: %s\n",
					       optarg);
				exit(EX_USAGE);
			}
			(void) smfi_setconn(optarg);
			setconn = true;
			break;

		}
	}
	if (!setconn)
	{
		fprintf(stderr, "%s: Missing required -p argument\n", argv[0]);
		exit(EX_USAGE);
	}
	/* if the below line does not comnpile, your libmilter is not suitably patched 
	 *
	 * See the milter-rrres patch http://www.jmaimon.com/sendmail
	 */
	smfilter.xxfi_flags |= SMFIF_REWRITE; 
	smfilter.xxfi_flags |= SMFIF_CHGHDRS; 
	smfilter.xxfi_flags |= SMFIF_ADDHDRS; 
	if (smfi_register(smfilter) == MI_FAILURE)
	{
		fprintf(stderr, "smfi_register failed\n");
		exit(EX_UNAVAILABLE);
	}
	return smfi_main();
}


